/** 
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.servicemix.components.jms;

import org.activemq.jca.JCAConnector;
import org.activemq.jca.JCAContainer;
import org.activemq.jca.SingletonEndpointFactory;
import org.springframework.beans.factory.InitializingBean;

import javax.resource.spi.ActivationSpec;
import javax.transaction.TransactionManager;

/**
 * Uses the JCA Container for better inbound subscription.
 *
 * @version $Revision: 1.2 $
 */
public class JmsInUsingJCABinding extends JmsInBinding implements InitializingBean {

    private JCAContainer jcaContainer;
    private ActivationSpec activationSpec;
    private TransactionManager transactionManager;
    private JCAConnector jcaConnector;

    public void afterPropertiesSet() throws Exception {
        if (jcaContainer == null) {
            throw new IllegalArgumentException("Must specify a jcaContainer property");
        }
        if (activationSpec == null) {
            throw new IllegalArgumentException("Must specify an activationSpec property");
        }
        jcaConnector = jcaContainer.addConnector();
        jcaConnector.setActivationSpec(activationSpec);
        if (transactionManager != null) {
            jcaConnector.setTransactionManager(transactionManager);
        }
        jcaConnector.setEndpointFactory(new SingletonEndpointFactory(this));
        jcaConnector.afterPropertiesSet();
    }

    public JCAContainer getJcaContainer() {
        return jcaContainer;
    }

    public void setJcaContainer(JCAContainer jcaContainer) {
        this.jcaContainer = jcaContainer;
    }

    public ActivationSpec getActivationSpec() {
        return activationSpec;
    }

    public void setActivationSpec(ActivationSpec activationSpec) {
        this.activationSpec = activationSpec;
    }

    public TransactionManager getTransactionManager() {
        return transactionManager;
    }

    public void setTransactionManager(TransactionManager transactionManager) {
        this.transactionManager = transactionManager;
    }

    public JCAConnector getJcaConnector() {
        return jcaConnector;
    }
}
