/** 
 * 
 * Copyright 2005 LogicBlaze, Inc. http://www.logicblaze.com
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License. 
 * 
 **/
package org.servicemix.components.util;

import org.servicemix.MessageExchangeListener;
import org.servicemix.jbi.NoInMessageAvailableException;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;

/**
 * A useful base class for a transform component.
 *
 * @version $Revision: 1.5 $
 */
public abstract class TransformComponentSupport extends ComponentSupport implements MessageExchangeListener {
    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
        NormalizedMessage in = exchange.getMessage("in");
        if (in == null) {
            throw new NoInMessageAvailableException(exchange);
        }
        NormalizedMessage out = exchange.createMessage();
        try {
            if (transform(exchange, in, out)) {
                if (isInAndOut(exchange)) {
                    exchange.setMessage(out, "out");
                }
                else {
                    InOnly outExchange = getExchangeFactory().createInOnlyExchange();
                    outExchange.setInMessage(out);
                    getDeliveryChannel().sendSync(outExchange);
                }
            }
            if (exchange.getFault() == null) {
                done(exchange);
            }
            else {
                exchange.setStatus(ExchangeStatus.ERROR);
                getDeliveryChannel().send(exchange);
            }
        }
        catch (MessagingException e) {
            fail(exchange, e);
        }
    }


    // Implementation methods
    //-------------------------------------------------------------------------

    /**
     * Transforms the given out message
     */
    protected abstract boolean transform(MessageExchange exchange, NormalizedMessage in, NormalizedMessage out) throws MessagingException;
}
